@@ -158,14 +158,14 @@ var SyscallsLibrary = {
       SYSCALLS.varargs += 4;
       var ret = {{{ makeGetValue('SYSCALLS.varargs', '-4', 'i32') }}};
 #if SYSCALL_DEBUG
-      Module.printErr('    (raw: "' + ret + '")');
+      err('    (raw: "' + ret + '")');
 #endif
       return ret;
     },
     getStr: function() {
       var ret = Pointer_stringify(SYSCALLS.get());
 #if SYSCALL_DEBUG
-      Module.printErr('    (str: "' + ret + '")');
+      err('    (str: "' + ret + '")');
 #endif
       return ret;
     },
@@ -174,7 +174,7 @@ var SyscallsLibrary = {
       var stream = FS.getStream(SYSCALLS.get());
       if (!stream) throw new FS.ErrnoError(ERRNO_CODES.EBADF);
 #if SYSCALL_DEBUG
-      Module.printErr('    (stream: "' + stream.path + '")');
+      err('    (stream: "' + stream.path + '")');
 #endif
       return stream;
     },
@@ -182,7 +182,7 @@ var SyscallsLibrary = {
       var socket = SOCKFS.getSocket(SYSCALLS.get());
       if (!socket) throw new FS.ErrnoError(ERRNO_CODES.EBADF);
 #if SYSCALL_DEBUG
-      Module.printErr('    (socket: "' + socket.path + '")');
+      err('    (socket: "' + socket.path + '")');
 #endif
       return socket;
     },
@@ -193,7 +193,7 @@ var SyscallsLibrary = {
       if (info.errno) throw new FS.ErrnoError(info.errno);
       info.addr = DNS.lookup_addr(info.addr) || info.addr;
 #if SYSCALL_DEBUG
-      Module.printErr('    (socketaddress: "' + [info.addr, info.port] + '")');
+      err('    (socketaddress: "' + [info.addr, info.port] + '")');
 #endif
       return info;
     },
@@ -203,7 +203,7 @@ var SyscallsLibrary = {
       if (low >= 0) assert(high === 0);
       else assert(high === -1);
 #if SYSCALL_DEBUG
-      Module.printErr('    (i64: "' + low + '")');
+      err('    (i64: "' + low + '")');
 #endif
       return low;
     },
@@ -214,7 +214,7 @@ var SyscallsLibrary = {
 
   __syscall1: function(which, varargs) { // exit
     var status = SYSCALLS.get();
-    Module['exit'](status);
+    exit(status);
     return 0;
   },
   __syscall3: function(which, varargs) { // read
@@ -315,20 +315,26 @@ var SyscallsLibrary = {
   __syscall54: function(which, varargs) { // ioctl
 #if NO_FILESYSTEM
 #if SYSCALL_DEBUG
-    Module.printErr('no-op in ioctl syscall due to NO_FILESYSTEM');
+    err('no-op in ioctl syscall due to NO_FILESYSTEM');
 #endif
     return 0;
 #else
     var stream = SYSCALLS.getStreamFromFD(), op = SYSCALLS.get();
     switch (op) {
+      case {{{ cDefine('TCGETA') }}}:
       case {{{ cDefine('TCGETS') }}}: {
         if (!stream.tty) return -ERRNO_CODES.ENOTTY;
 #if SYSCALL_DEBUG
-        Module.printErr('warning: not filling tio struct');
+        err('warning: not filling tio struct');
 #endif
         return 0;
       }
-      case {{{ cDefine('TCSETS') }}}: {
+      case {{{ cDefine('TCSETA') }}}:
+      case {{{ cDefine('TCSETAW') }}}:
+      case {{{ cDefine('TCSETAF') }}}:
+      case {{{ cDefine('TCSETS') }}}:
+      case {{{ cDefine('TCSETSW') }}}:
+      case {{{ cDefine('TCSETSF') }}}: {
         if (!stream.tty) return -ERRNO_CODES.ENOTTY;
         return 0; // no-op, not actually adjusting terminal settings
       }
@@ -352,6 +358,13 @@ var SyscallsLibrary = {
         if (!stream.tty) return -ERRNO_CODES.ENOTTY;
         return 0;
       }
+      case {{{ cDefine('TIOCSWINSZ') }}}: {
+        // TODO: technically, this ioctl call should change the window size.
+        // but, since emscripten doesn't have any concept of a terminal window
+        // yet, we'll just silently throw it away as we do TIOCGWINSZ
+        if (!stream.tty) return -ERRNO_CODES.ENOTTY;
+        return 0;
+      }
       default: abort('bad ioctl syscall ' + op);
     }
 #endif // NO_FILESYSTEM
@@ -390,7 +403,7 @@ var SyscallsLibrary = {
   },
   __syscall77: function(which, varargs) { // getrusage
 #if SYSCALL_DEBUG
-    Module.printErr('warning: untested syscall');
+    err('warning: untested syscall');
 #endif
     var who = SYSCALLS.get(), usage = SYSCALLS.get();
     _memset(usage, 0, {{{ C_STRUCTS.rusage.__size__ }}});
@@ -779,7 +792,20 @@ var SyscallsLibrary = {
     return SYSCALLS.doReadv(stream, iov, iovcnt);
   },
 #if NO_FILESYSTEM
-  __syscall146__postset: '/* flush anything remaining in the buffer during shutdown */ __ATEXIT__.push(function() { var fflush = Module["_fflush"]; if (fflush) fflush(0); var printChar = ___syscall146.printChar; if (!printChar) return; var buffers = ___syscall146.buffers; if (buffers[1].length) printChar(1, {{{ charCode("\n") }}}); if (buffers[2].length) printChar(2, {{{ charCode("\n") }}}); });',
+  $flush_NO_FILESYSTEM: function() {
+    // flush anything remaining in the buffers during shutdown
+    var fflush = Module["_fflush"];
+    if (fflush) fflush(0);
+    var printChar = ___syscall146.printChar;
+    if (!printChar) return;
+    var buffers = ___syscall146.buffers;
+    if (buffers[1].length) printChar(1, {{{ charCode("\n") }}});
+    if (buffers[2].length) printChar(2, {{{ charCode("\n") }}});
+  },
+  __syscall146__deps: ['$flush_NO_FILESYSTEM'],
+#if NO_EXIT_RUNTIME == 0
+  __syscall146__postset: '__ATEXIT__.push(flush_NO_FILESYSTEM);',
+#endif
 #endif
   __syscall146: function(which, varargs) { // writev
 #if NO_FILESYSTEM == 0
@@ -789,13 +815,13 @@ var SyscallsLibrary = {
     // hack to support printf in NO_FILESYSTEM
     var stream = SYSCALLS.get(), iov = SYSCALLS.get(), iovcnt = SYSCALLS.get();
     var ret = 0;
-    if (!___syscall146.buffer) {
+    if (!___syscall146.buffers) {
       ___syscall146.buffers = [null, [], []]; // 1 => stdout, 2 => stderr
       ___syscall146.printChar = function(stream, curr) {
         var buffer = ___syscall146.buffers[stream];
         assert(buffer);
         if (curr === 0 || curr === {{{ charCode('\n') }}}) {
-          (stream === 1 ? Module['print'] : Module['printErr'])(UTF8ArrayToString(buffer, 0));
+          (stream === 1 ? out : err)(UTF8ArrayToString(buffer, 0));
           buffer.length = 0;
         } else {
           buffer.push(curr);
@@ -855,7 +881,7 @@ var SyscallsLibrary = {
   },
   __syscall178: function(which, varargs) { // rt_sigqueueinfo
 #if SYSCALL_DEBUG
-    Module.printErr('warning: ignoring SYS_rt_sigqueueinfo');
+    err('warning: ignoring SYS_rt_sigqueueinfo');
 #endif
     return 0;
   },
@@ -871,13 +897,14 @@ var SyscallsLibrary = {
     var buf = SYSCALLS.get(), size = SYSCALLS.get();
     if (size === 0) return -ERRNO_CODES.EINVAL;
     var cwd = FS.cwd();
-    if (size < cwd.length + 1) return -ERRNO_CODES.ERANGE;
-    writeAsciiToMemory(cwd, buf);
+    var cwdLengthInBytes = lengthBytesUTF8(cwd);
+    if (size < cwdLengthInBytes + 1) return -ERRNO_CODES.ERANGE;
+    stringToUTF8(cwd, buf, size);
     return buf;
   },
   __syscall191: function(which, varargs) { // ugetrlimit
 #if SYSCALL_DEBUG
-    Module.printErr('warning: untested syscall');
+    err('warning: untested syscall');
 #endif
     var resource = SYSCALLS.get(), rlim = SYSCALLS.get();
     {{{ makeSetValue('rlim', C_STRUCTS.rlimit.rlim_cur, '-1', 'i32') }}};  // RLIM_INFINITY
@@ -972,7 +999,7 @@ var SyscallsLibrary = {
   __syscall209: '__syscall211',     // getresuid
   __syscall211: function(which, varargs) { // getresgid32
 #if SYSCALL_DEBUG
-    Module.printErr('warning: untested syscall');
+    err('warning: untested syscall');
 #endif
     var ruid = SYSCALLS.get(), euid = SYSCALLS.get(), suid = SYSCALLS.get();
     {{{ makeSetValue('ruid', '0', '0', 'i32') }}};
@@ -996,7 +1023,6 @@ var SyscallsLibrary = {
       var id;
       var type;
       var name = stream.getdents.pop();
-      assert(name.length < 256); // limit of dirent struct
       if (name[0] === '.') {
         id = 1;
         type = 4; // DT_DIR
@@ -1012,10 +1038,7 @@ var SyscallsLibrary = {
       {{{ makeSetValue('dirp + pos', C_STRUCTS.dirent.d_off, 'stream.position', 'i32') }}};
       {{{ makeSetValue('dirp + pos', C_STRUCTS.dirent.d_reclen, C_STRUCTS.dirent.__size__, 'i16') }}};
       {{{ makeSetValue('dirp + pos', C_STRUCTS.dirent.d_type, 'type', 'i8') }}};
-      for (var i = 0; i < name.length; i++) {
-        {{{ makeSetValue('dirp + pos', C_STRUCTS.dirent.d_name + ' + i', 'name.charCodeAt(i)', 'i8') }}};
-      }
-      {{{ makeSetValue('dirp + pos', C_STRUCTS.dirent.d_name + ' + i', '0', 'i8') }}};
+      stringToUTF8(name, dirp + pos + {{{ C_STRUCTS.dirent.d_name }}}, 256);
       pos += {{{ C_STRUCTS.dirent.__size__ }}};
     }
     return pos;
@@ -1024,7 +1047,7 @@ var SyscallsLibrary = {
   __syscall221: function(which, varargs) { // fcntl64
 #if NO_FILESYSTEM
 #if SYSCALL_DEBUG
-    Module.printErr('no-op in fcntl64 syscall due to NO_FILESYSTEM');
+    err('no-op in fcntl64 syscall due to NO_FILESYSTEM');
 #endif
     return 0;
 #else
@@ -1071,7 +1094,7 @@ var SyscallsLibrary = {
         return -1;
       default: {
 #if SYSCALL_DEBUG
-        Module.printErr('warning: fctl64 unrecognized command ' + cmd);
+        err('warning: fctl64 unrecognized command ' + cmd);
 #endif
         return -ERRNO_CODES.EINVAL;
       }
@@ -1080,7 +1103,7 @@ var SyscallsLibrary = {
   },
   __syscall265: function(which, varargs) { // clock_nanosleep
 #if SYSCALL_DEBUG
-    Module.printErr('warning: ignoring SYS_clock_nanosleep');
+    err('warning: ignoring SYS_clock_nanosleep');
 #endif
     return 0;
   },
@@ -1110,7 +1133,7 @@ var SyscallsLibrary = {
   },
   __syscall295: function(which, varargs) { // openat
 #if SYSCALL_DEBUG
-    Module.printErr('warning: untested syscall');
+    err('warning: untested syscall');
 #endif
     var dirfd = SYSCALLS.get(), path = SYSCALLS.getStr(), flags = SYSCALLS.get(), mode = SYSCALLS.get();
     path = SYSCALLS.calculateAt(dirfd, path);
@@ -1118,7 +1141,7 @@ var SyscallsLibrary = {
   },
   __syscall296: function(which, varargs) { // mkdirat
 #if SYSCALL_DEBUG
-    Module.printErr('warning: untested syscall');
+    err('warning: untested syscall');
 #endif
     var dirfd = SYSCALLS.get(), path = SYSCALLS.getStr(), mode = SYSCALLS.get();
     path = SYSCALLS.calculateAt(dirfd, path);
@@ -1126,7 +1149,7 @@ var SyscallsLibrary = {
   },
   __syscall297: function(which, varargs) { // mknodat
 #if SYSCALL_DEBUG
-    Module.printErr('warning: untested syscall');
+    err('warning: untested syscall');
 #endif
     var dirfd = SYSCALLS.get(), path = SYSCALLS.getStr(), mode = SYSCALLS.get(), dev = SYSCALLS.get();
     path = SYSCALLS.calculateAt(dirfd, path);
@@ -1134,7 +1157,7 @@ var SyscallsLibrary = {
   },
   __syscall298: function(which, varargs) { // fchownat
 #if SYSCALL_DEBUG
-    Module.printErr('warning: untested syscall');
+    err('warning: untested syscall');
 #endif
     var dirfd = SYSCALLS.get(), path = SYSCALLS.getStr(), owner = SYSCALLS.get(), group = SYSCALLS.get(), flags = SYSCALLS.get();
     assert(flags === 0);
@@ -1155,17 +1178,22 @@ var SyscallsLibrary = {
   },
   __syscall301: function(which, varargs) { // unlinkat
 #if SYSCALL_DEBUG
-    Module.printErr('warning: untested syscall');
+    err('warning: untested syscall');
 #endif
     var dirfd = SYSCALLS.get(), path = SYSCALLS.getStr(), flags = SYSCALLS.get();
-    assert(flags === 0);
     path = SYSCALLS.calculateAt(dirfd, path);
-    FS.unlink(path);
+    if (flags === 0) {
+      FS.unlink(path);
+    } else if (flags === {{{ cDefine('AT_REMOVEDIR') }}}) {
+      FS.rmdir(path);
+    } else {
+      abort('Invalid flags passed to unlinkat');
+    }
     return 0;
   },
   __syscall302: function(which, varargs) { // renameat
 #if SYSCALL_DEBUG
-    Module.printErr('warning: untested syscall');
+    err('warning: untested syscall');
 #endif
     var olddirfd = SYSCALLS.get(), oldpath = SYSCALLS.getStr(), newdirfd = SYSCALLS.get(), newpath = SYSCALLS.getStr();
     oldpath = SYSCALLS.calculateAt(olddirfd, oldpath);
@@ -1178,7 +1206,7 @@ var SyscallsLibrary = {
   },
   __syscall304: function(which, varargs) { // symlinkat
 #if SYSCALL_DEBUG
-    Module.printErr('warning: untested syscall');
+    err('warning: untested syscall');
 #endif
     var target = SYSCALLS.get(), newdirfd = SYSCALLS.get(), linkpath = SYSCALLS.get();
     linkpath = SYSCALLS.calculateAt(newdirfd, linkpath);
@@ -1187,7 +1215,7 @@ var SyscallsLibrary = {
   },
   __syscall305: function(which, varargs) { // readlinkat
 #if SYSCALL_DEBUG
-    Module.printErr('warning: untested syscall');
+    err('warning: untested syscall');
 #endif
     var dirfd = SYSCALLS.get(), path = SYSCALLS.getStr(), buf = SYSCALLS.get(), bufsize = SYSCALLS.get();
     path = SYSCALLS.calculateAt(dirfd, path);
@@ -1195,7 +1223,7 @@ var SyscallsLibrary = {
   },
   __syscall306: function(which, varargs) { // fchmodat
 #if SYSCALL_DEBUG
-    Module.printErr('warning: untested syscall');
+    err('warning: untested syscall');
 #endif
     var dirfd = SYSCALLS.get(), path = SYSCALLS.getStr(), mode = SYSCALLS.get(), flags = SYSCALLS.get();
     assert(flags === 0);
@@ -1205,7 +1233,7 @@ var SyscallsLibrary = {
   },
   __syscall307: function(which, varargs) { // faccessat
 #if SYSCALL_DEBUG
-    Module.printErr('warning: untested syscall');
+    err('warning: untested syscall');
 #endif
     var dirfd = SYSCALLS.get(), path = SYSCALLS.getStr(), amode = SYSCALLS.get(), flags = SYSCALLS.get();
     assert(flags === 0);
@@ -1217,7 +1245,7 @@ var SyscallsLibrary = {
   },
   __syscall320: function(which, varargs) { // utimensat
 #if SYSCALL_DEBUG
-    Module.printErr('warning: untested syscall');
+    err('warning: untested syscall');
 #endif
     var dirfd = SYSCALLS.get(), path = SYSCALLS.getStr(), times = SYSCALLS.get(), flags = SYSCALLS.get();
     assert(flags === 0);
@@ -1240,7 +1268,7 @@ var SyscallsLibrary = {
   },
   __syscall330: function(which, varargs) { // dup3
 #if SYSCALL_DEBUG
-    Module.printErr('warning: untested syscall');
+    err('warning: untested syscall');
 #endif
     var old = SYSCALLS.getStreamFromFD(), suggestFD = SYSCALLS.get(), flags = SYSCALLS.get();
     assert(!flags);
@@ -1252,21 +1280,21 @@ var SyscallsLibrary = {
   },
   __syscall333: function(which, varargs) { // preadv
 #if SYSCALL_DEBUG
-    Module.printErr('warning: untested syscall');
+    err('warning: untested syscall');
 #endif
     var stream = SYSCALLS.getStreamFromFD(), iov = SYSCALLS.get(), iovcnt = SYSCALLS.get(), offset = SYSCALLS.get();
     return SYSCALLS.doReadv(stream, iov, iovcnt, offset);
   },
   __syscall334: function(which, varargs) { // pwritev
 #if SYSCALL_DEBUG
-    Module.printErr('warning: untested syscall');
+    err('warning: untested syscall');
 #endif
     var stream = SYSCALLS.getStreamFromFD(), iov = SYSCALLS.get(), iovcnt = SYSCALLS.get(), offset = SYSCALLS.get();
     return SYSCALLS.doWritev(stream, iov, iovcnt, offset);
   },
   __syscall337: function(which, varargs) { // recvmmsg
 #if SYSCALL_DEBUG
-    Module.printErr('warning: ignoring SYS_recvmmsg');
+    err('warning: ignoring SYS_recvmmsg');
 #endif
     return 0;
   },
@@ -1282,7 +1310,7 @@ var SyscallsLibrary = {
   },
   __syscall345: function(which, varargs) { // sendmmsg
 #if SYSCALL_DEBUG
-    Module.printErr('warning: ignoring SYS_sendmmsg');
+    err('warning: ignoring SYS_sendmmsg');
 #endif
     return 0;
   },
@@ -1654,14 +1682,14 @@ for (var x in SyscallsLibrary) {
 #endif
   pre += 'SYSCALLS.varargs = varargs;\n';
 #if SYSCALL_DEBUG
-  pre += "Module.printErr('syscall! ' + [" + which + ", '" + SYSCALL_CODE_TO_NAME[which] + "']);\n";
+  pre += "err('syscall! ' + [" + which + ", '" + SYSCALL_CODE_TO_NAME[which] + "']);\n";
   pre += "var canWarn = true;\n";
   pre += "var ret = (function() {\n";
   post += "})();\n";
   post += "if (ret < 0 && canWarn) {\n";
-  post += "  Module.printErr('error: syscall may have failed with ' + (-ret) + ' (' + ERRNO_MESSAGES[-ret] + ')');\n";
+  post += "  err('error: syscall may have failed with ' + (-ret) + ' (' + ERRNO_MESSAGES[-ret] + ')');\n";
   post += "}\n";
-  post += "Module.printErr('syscall return: ' + ret);\n";
+  post += "err('syscall return: ' + ret);\n";
   post += "return ret;\n";
 #endif
   pre += 'try {\n';
@@ -1670,7 +1698,7 @@ for (var x in SyscallsLibrary) {
   "  if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);\n";
 #if SYSCALL_DEBUG
   handler +=
-  "  Module.printErr('error: syscall failed with ' + e.errno + ' (' + ERRNO_MESSAGES[e.errno] + ')');\n" +
+  "  err('error: syscall failed with ' + e.errno + ' (' + ERRNO_MESSAGES[e.errno] + ')');\n" +
   "  canWarn = false;\n";
 #endif
   handler +=
