@@ -46,8 +46,8 @@ Module['quit'] = function(status, toThrow) {
 Module['preRun'] = [];
 Module['postRun'] = [];
 
-// The environment setup code below is customized to use Module.
-// *** Environment setup code ***
+// Determine the runtime environment we are in. You can customize this by
+// setting the ENVIRONMENT setting at compile time (see settings.js).
 
 #if ENVIRONMENT
 var ENVIRONMENT_IS_WEB = {{{ ENVIRONMENT === 'web' }}};
@@ -230,18 +230,29 @@ if (ENVIRONMENT_IS_SHELL) {
 #if ENVIRONMENT_MAY_BE_WEB_OR_WORKER
 if (ENVIRONMENT_IS_WEB || ENVIRONMENT_IS_WORKER) {
   if (ENVIRONMENT_IS_WEB) {
-#if MODULARIZE
-    // When MODULARIZE, this JS may be executed later, after document.currentScript is gone, so we send it
-    // using this._currentScript.
-    var currentScript = this['_currentScript'] || document.currentScript;
-#else
-    var currentScript = document.currentScript;
-#endif
-    if (currentScript.src.indexOf('blob:') !== 0) {
-      scriptDirectory = currentScript.src.split('/').slice(0, -1).join('/') + '/';
+    if (document.currentScript) {
+      scriptDirectory = document.currentScript.src;
     }
-  } else if (ENVIRONMENT_IS_WORKER) {
-    scriptDirectory = self.location.href.split('/').slice(0, -1).join('/') + '/';
+  } else { // worker
+    scriptDirectory = self.location.href;
+  }
+#if MODULARIZE
+#if MODULARIZE_INSTANCE == 0
+  // When MODULARIZE (and not _INSTANCE), this JS may be executed later, after document.currentScript
+  // is gone, so we saved it, and we use it here instead of any other info.
+  if (_scriptDir) {
+    scriptDirectory = _scriptDir;
+  }
+#endif
+#endif
+  // blob urls look like blob:http://site.com/etc/etc and we cannot infer anything from them.
+  // otherwise, slice off the final part of the url to find the script directory.
+  // if scriptDirectory does not contain a slash, lastIndexOf will return -1,
+  // and scriptDirectory will correctly be replaced with an empty string.
+  if (scriptDirectory.indexOf('blob:') !== 0) {
+    scriptDirectory = scriptDirectory.substr(0, scriptDirectory.lastIndexOf('/')+1);
+  } else {
+    scriptDirectory = '';
   }
 
 #if ENVIRONMENT
@@ -331,8 +342,6 @@ if (ENVIRONMENT_IS_WEB || ENVIRONMENT_IS_WORKER) {
 var out = Module['print'] || (typeof console !== 'undefined' ? console.log.bind(console) : (typeof print !== 'undefined' ? print : null));
 var err = Module['printErr'] || (typeof printErr !== 'undefined' ? printErr : ((typeof console !== 'undefined' && console.warn.bind(console)) || out));
 
-// *** Environment setup code ***
-
 // Merge back in the overrides
 for (key in moduleOverrides) {
   if (moduleOverrides.hasOwnProperty(key)) {
