@@ -262,7 +262,7 @@ static size_t getFreeListIndex(size_t size) {
   size_t index = lowerBoundPowerOf2(size);
   assert(MIN_FREELIST_INDEX <= index && index < MAX_FREELIST_INDEX);
 #ifdef EMMALLOC_DEBUG_LOG
-  EM_ASM({ Module.print("  emmalloc.getFreeListIndex " + [$0, $1]) }, size, index);
+  EM_ASM({ out("  emmalloc.getFreeListIndex " + [$0, $1]) }, size, index);
 #endif
   return index;
 }
@@ -277,7 +277,7 @@ static size_t getBigEnoughFreeListIndex(size_t size) {
   // same. Otherwise, add one.
   if (!isPowerOf2(size)) index++;
 #ifdef EMMALLOC_DEBUG_LOG
-  EM_ASM({ Module.print("  emmalloc.getBigEnoughFreeListIndex " + [$0, $1]) }, size, index);
+  EM_ASM({ out("  emmalloc.getBigEnoughFreeListIndex " + [$0, $1]) }, size, index);
 #endif
   return index;
 }
@@ -294,7 +294,7 @@ static size_t getMaxSizeForFreeListIndex(size_t index) {
 
 static void removeFromFreeList(Region* region) {
 #ifdef EMMALLOC_DEBUG_LOG
-  EM_ASM({ Module.print("  emmalloc.removeFromFreeList " + $0) },region);
+  EM_ASM({ out("  emmalloc.removeFromFreeList " + $0) },region);
 #endif
   size_t index = getFreeListIndex(getMaxPayload(region));
   FreeInfo* freeInfo = &region->freeInfo();
@@ -311,7 +311,7 @@ static void removeFromFreeList(Region* region) {
 
 static void addToFreeList(Region* region) {
 #ifdef EMMALLOC_DEBUG_LOG
-  EM_ASM({ Module.print("  emmalloc.addToFreeList " + $0) }, region);
+  EM_ASM({ out("  emmalloc.addToFreeList " + $0) }, region);
 #endif
   assert(getAfter(region) <= sbrk(0));
   size_t index = getFreeListIndex(getMaxPayload(region));
@@ -329,7 +329,7 @@ static void addToFreeList(Region* region) {
 // Tries to merge it into a region before or after it to which it is adjacent.
 static int mergeIntoExistingFreeRegion(Region* region) {
 #ifdef EMMALLOC_DEBUG_LOG
-  EM_ASM({ Module.print("  emmalloc.mergeIntoExistingFreeRegion " + $0) }, region);
+  EM_ASM({ out("  emmalloc.mergeIntoExistingFreeRegion " + $0) }, region);
 #endif
   assert(getAfter(region) <= sbrk(0));
   int merged = 0;
@@ -338,7 +338,7 @@ static int mergeIntoExistingFreeRegion(Region* region) {
   if (prev && !prev->getUsed()) {
     // Merge them.
 #ifdef EMMALLOC_DEBUG_LOG
-    EM_ASM({ Module.print("  emmalloc.mergeIntoExistingFreeRegion merge into prev " + $0) }, prev);
+    EM_ASM({ out("  emmalloc.mergeIntoExistingFreeRegion merge into prev " + $0) }, prev);
 #endif
     removeFromFreeList(prev);
     prev->incTotalSize(region->getTotalSize());
@@ -352,7 +352,7 @@ static int mergeIntoExistingFreeRegion(Region* region) {
       // We may also be able to merge with the next, keep trying.
       if (!next->getUsed()) {
 #ifdef EMMALLOC_DEBUG_LOG
-        EM_ASM({ Module.print("  emmalloc.mergeIntoExistingFreeRegion also merge into next " + $0) }, next);
+        EM_ASM({ out("  emmalloc.mergeIntoExistingFreeRegion also merge into next " + $0) }, next);
 #endif
         removeFromFreeList(next);
         prev->incTotalSize(next->getTotalSize());
@@ -368,7 +368,7 @@ static int mergeIntoExistingFreeRegion(Region* region) {
   }
   if (next && !next->getUsed()) {
 #ifdef EMMALLOC_DEBUG_LOG
-    EM_ASM({ Module.print("  emmalloc.mergeIntoExistingFreeRegion merge into next " + $0) }, next);
+    EM_ASM({ out("  emmalloc.mergeIntoExistingFreeRegion merge into next " + $0) }, next);
 #endif
     // Merge them.
     removeFromFreeList(next);
@@ -396,7 +396,7 @@ static void stopUsing(Region* region) {
 // TODO: We can calculate that, to save some work.
 static void growRegion(Region* region, size_t sizeDelta) {
 #ifdef EMMALLOC_DEBUG_LOG
-  EM_ASM({ Module.print("  emmalloc.growRegion " + [$0, $1]) }, region, sizeDelta);
+  EM_ASM({ out("  emmalloc.growRegion " + [$0, $1]) }, region, sizeDelta);
 #endif
   if (!region->getUsed()) {
     removeFromFreeList(region);
@@ -411,7 +411,7 @@ static void growRegion(Region* region, size_t sizeDelta) {
 // 0 if an error occurred in sbrk().
 static int extendLastRegion(size_t size) {
 #ifdef EMMALLOC_DEBUG_LOG
-  EM_ASM({ Module.print("  emmalloc.extendLastRegionToSize " + $0) }, size);
+  EM_ASM({ out("  emmalloc.extendLastRegionToSize " + $0) }, size);
 #endif
   size_t reusable = getMaxPayload(lastRegion);
   size_t sbrkSize = alignUp(size) - reusable;
@@ -419,7 +419,7 @@ static int extendLastRegion(size_t size) {
   if (ptr == (void*)-1) {
     // sbrk() failed, we failed.
 #ifdef EMMALLOC_DEBUG_LOG
-   EM_ASM({ Module.print("  emmalloc.extendLastRegion sbrk failure") });
+   EM_ASM({ out("  emmalloc.extendLastRegion sbrk failure") });
 #endif
     return 0;
   }
@@ -432,7 +432,7 @@ static int extendLastRegion(size_t size) {
 
 static void possiblySplitRemainder(Region* region, size_t size) {
 #ifdef EMMALLOC_DEBUG_LOG
-  EM_ASM({ Module.print("  emmalloc.possiblySplitRemainder " + [$0, $1]) }, region, size);
+  EM_ASM({ out("  emmalloc.possiblySplitRemainder " + [$0, $1]) }, region, size);
 #endif
   size_t payloadSize = getMaxPayload(region);
   assert(payloadSize >= size);
@@ -447,7 +447,7 @@ static void possiblySplitRemainder(Region* region, size_t size) {
     // Yes, this is a small-but-useful amount of memory in the final region,
     // extend it.
 #ifdef EMMALLOC_DEBUG_LOG
-    EM_ASM({ Module.print("    emmalloc.possiblySplitRemainder pre-extending") });
+    EM_ASM({ out("    emmalloc.possiblySplitRemainder pre-extending") });
 #endif
     if (extendLastRegion(payloadSize + ALLOC_UNIT)) {
       // Success.
@@ -459,7 +459,7 @@ static void possiblySplitRemainder(Region* region, size_t size) {
   }
   if (extra >= MIN_REGION_SIZE) {
 #ifdef EMMALLOC_DEBUG_LOG
-    EM_ASM({ Module.print("    emmalloc.possiblySplitRemainder is splitting") });
+    EM_ASM({ out("    emmalloc.possiblySplitRemainder is splitting") });
 #endif
     // Worth it, split the region
     // TODO: Consider not doing it, may affect long-term fragmentation.
@@ -484,7 +484,7 @@ static void possiblySplitRemainder(Region* region, size_t size) {
 // any.
 static void useRegion(Region* region, size_t size) {
 #ifdef EMMALLOC_DEBUG_LOG
-  EM_ASM({ Module.print("  emmalloc.useRegion " + [$0, $1]) }, region, size);
+  EM_ASM({ out("  emmalloc.useRegion " + [$0, $1]) }, region, size);
 #endif
   assert(size > 0);
   region->setUsed(1);
@@ -496,7 +496,7 @@ static void useRegion(Region* region, size_t size) {
 static Region* useFreeInfo(FreeInfo* freeInfo, size_t size) {
   Region* region = fromFreeInfo(freeInfo);
 #ifdef EMMALLOC_DEBUG_LOG
-  EM_ASM({ Module.print("  emmalloc.useFreeInfo " + [$0, $1]) }, region, size);
+  EM_ASM({ out("  emmalloc.useFreeInfo " + [$0, $1]) }, region, size);
 #endif
   // This region is no longer free
   removeFromFreeList(region);
@@ -602,15 +602,15 @@ static void emmalloc_validate_all() {
 #ifdef EMMALLOC_DEBUG_LOG
 // For testing purposes, dump out a region.
 static void emmalloc_dump_region(Region* region) {
-  EM_ASM({ Module.print("      [" + $0 + " - " + $1 + " (" + $2 + " bytes" + ($3 ? ", used" : "") + ")]") },
+  EM_ASM({ out("      [" + $0 + " - " + $1 + " (" + $2 + " bytes" + ($3 ? ", used" : "") + ")]") },
          region, getAfter(region), getMaxPayload(region), region->getUsed());
 }
 
 // For testing purposes, dumps out the entire global state.
 static void emmalloc_dump_all() {
-  EM_ASM({ Module.print("  emmalloc_dump_all:\n    sbrk(0) = " + $0) }, sbrk(0));
+  EM_ASM({ out("  emmalloc_dump_all:\n    sbrk(0) = " + $0) }, sbrk(0));
   Region* curr = firstRegion;
-  EM_ASM({ Module.print("    all regions:") });
+  EM_ASM({ out("    all regions:") });
   while (curr) {
     emmalloc_dump_region(curr);
     curr = curr->next();
@@ -618,7 +618,7 @@ static void emmalloc_dump_all() {
   for (int i = 0; i < MAX_FREELIST_INDEX; i++) {
     FreeInfo* curr = freeLists[i];
     if (!curr) continue;
-    EM_ASM({ Module.print("    freeList[" + $0 + "] sizes: [" + $1 + ", " + $2 + ")") }, i, getMinSizeForFreeListIndex(i), getMaxSizeForFreeListIndex(i));
+    EM_ASM({ out("    freeList[" + $0 + "] sizes: [" + $1 + ", " + $2 + ")") }, i, getMinSizeForFreeListIndex(i), getMaxSizeForFreeListIndex(i));
     FreeInfo* prev = nullptr;
     while (curr) {
       Region* region = fromFreeInfo(curr);
@@ -659,7 +659,7 @@ static const size_t SPECULATIVE_FREELIST_TRIES = 32;
 
 static Region* tryFromFreeList(size_t size) {
 #ifdef EMMALLOC_DEBUG_LOG
-  EM_ASM({ Module.print("  emmalloc.tryFromFreeList " + $0) }, size);
+  EM_ASM({ out("  emmalloc.tryFromFreeList " + $0) }, size);
 #endif
   // Look in the freelist of items big enough for us.
   size_t index = getBigEnoughFreeListIndex(size);
@@ -676,7 +676,7 @@ static Region* tryFromFreeList(size_t size) {
       if (getMaxPayload(region) >= size) {
         // Success, use it
 #ifdef EMMALLOC_DEBUG_LOG
-        EM_ASM({ Module.print("  emmalloc.tryFromFreeList try succeeded") });
+        EM_ASM({ out("  emmalloc.tryFromFreeList try succeeded") });
 #endif
         return useFreeInfo(freeInfo, size);
       }
@@ -696,7 +696,7 @@ static Region* tryFromFreeList(size_t size) {
     if (freeInfo) {
       // We found one, use it.
 #ifdef EMMALLOC_DEBUG_LOG
-      EM_ASM({ Module.print("  emmalloc.tryFromFreeList had item to use") });
+      EM_ASM({ out("  emmalloc.tryFromFreeList had item to use") });
 #endif
       return useFreeInfo(freeInfo, size);
     }
@@ -707,7 +707,7 @@ static Region* tryFromFreeList(size_t size) {
   }
   // No luck, no free list.
 #ifdef EMMALLOC_DEBUG_LOG
-  EM_ASM({ Module.print("  emmalloc.tryFromFreeList no luck") });
+  EM_ASM({ out("  emmalloc.tryFromFreeList no luck") });
 #endif
   return nullptr;
 }
@@ -715,14 +715,14 @@ static Region* tryFromFreeList(size_t size) {
 // Allocate a completely new region.
 static Region* allocateRegion(size_t size) {
 #ifdef EMMALLOC_DEBUG_LOG
-  EM_ASM({ Module.print("  emmalloc.allocateRegion") });
+  EM_ASM({ out("  emmalloc.allocateRegion") });
 #endif
   size_t sbrkSize = METADATA_SIZE + alignUp(size);
   void* ptr = sbrk(sbrkSize);
   if (ptr == (void*)-1) {
     // sbrk() failed, we failed.
 #ifdef EMMALLOC_DEBUG_LOG
-    EM_ASM({ Module.print("    emmalloc.allocateRegion sbrk failure") });
+    EM_ASM({ out("    emmalloc.allocateRegion sbrk failure") });
 #endif
     return nullptr;
   }
@@ -731,14 +731,14 @@ static Region* allocateRegion(size_t size) {
   void* fixedPtr = alignUpPointer(ptr);
   if (ptr != fixedPtr) {
 #ifdef EMMALLOC_DEBUG_LOG
-    EM_ASM({ Module.print("    emmalloc.allocateRegion fixing alignment") });
+    EM_ASM({ out("    emmalloc.allocateRegion fixing alignment") });
 #endif
     size_t extra = (char*)fixedPtr - (char*)ptr;
     void* extraPtr = sbrk(extra);
     if (extraPtr == (void*)-1) {
       // sbrk() failed, we failed.
 #ifdef EMMALLOC_DEBUG_LOG
-      EM_ASM({ Module.print("    emmalloc.newAllocation sbrk failure") });;
+      EM_ASM({ out("    emmalloc.newAllocation sbrk failure") });;
 #endif
       return nullptr;
     }
@@ -766,7 +766,7 @@ static Region* allocateRegion(size_t size) {
   }
   // Success, we have new memory
 #ifdef EMMALLOC_DEBUG_LOG
-  EM_ASM({ Module.print("    emmalloc.newAllocation success") });;
+  EM_ASM({ out("    emmalloc.newAllocation success") });;
 #endif
   region->setTotalSize(sbrkSize);
   region->setUsed(1);
@@ -777,7 +777,7 @@ static Region* allocateRegion(size_t size) {
 // allocating what we need.
 static Region* newAllocation(size_t size) {
 #ifdef EMMALLOC_DEBUG_LOG
-  EM_ASM({ Module.print("  emmalloc.newAllocation " + $0) }, size);
+  EM_ASM({ out("  emmalloc.newAllocation " + $0) }, size);
 #endif
   assert(size > 0);
   if (lastRegion) {
@@ -786,7 +786,7 @@ static Region* newAllocation(size_t size) {
     // more efficient and simple as well.
     if (!lastRegion->getUsed()) {
 #ifdef EMMALLOC_DEBUG_LOG
-      EM_ASM({ Module.print("    emmalloc.newAllocation extending lastRegion at " + $0) }, lastRegion);
+      EM_ASM({ out("    emmalloc.newAllocation extending lastRegion at " + $0) }, lastRegion);
 #endif
       // Remove it first, before we adjust the size (which affects which list
       // it should be in). Also mark it as used so extending it doesn't do
@@ -849,7 +849,7 @@ static void* emmalloc_realloc(void *ptr, size_t size) {
   // Grow it. First, maybe we can do simple growth in the current region.
   if (size <= getMaxPayload(region)) {
 #ifdef EMMALLOC_DEBUG_LOG
-    EM_ASM({ Module.print("  emmalloc.emmalloc_realloc use existing payload space") });
+    EM_ASM({ out("  emmalloc.emmalloc_realloc use existing payload space") });
 #endif
     region->setUsed(1);
     // There might be enough left over to split out now.
@@ -860,7 +860,7 @@ static void* emmalloc_realloc(void *ptr, size_t size) {
   Region* next = region->next();
   if (next && !next->getUsed()) {
 #ifdef EMMALLOC_DEBUG_LOG
-    EM_ASM({ Module.print("  emmalloc.emmalloc_realloc merge in next") });
+    EM_ASM({ out("  emmalloc.emmalloc_realloc merge in next") });
 #endif
     removeFromFreeList(next);
     region->incTotalSize(next->getTotalSize());
@@ -873,7 +873,7 @@ static void* emmalloc_realloc(void *ptr, size_t size) {
   // We may now be big enough.
   if (size <= getMaxPayload(region)) {
 #ifdef EMMALLOC_DEBUG_LOG
-    EM_ASM({ Module.print("  emmalloc.emmalloc_realloc use existing payload space after merge") });
+    EM_ASM({ out("  emmalloc.emmalloc_realloc use existing payload space after merge") });
 #endif
     region->setUsed(1);
     // There might be enough left over to split out now.
@@ -887,7 +887,7 @@ static void* emmalloc_realloc(void *ptr, size_t size) {
   Region* newRegion = tryFromFreeList(size);
   if (!newRegion && region == lastRegion) {
 #ifdef EMMALLOC_DEBUG_LOG
-    EM_ASM({ Module.print("  emmalloc.emmalloc_realloc extend last region") });
+    EM_ASM({ out("  emmalloc.emmalloc_realloc extend last region") });
 #endif
     if (extendLastRegion(size)) {
       // It worked. We don't need the formerly free region.
@@ -944,7 +944,7 @@ static struct mallinfo emmalloc_mallinfo() {
 // large allocations.
 static void* alignedAllocation(size_t size, size_t alignment) {
 #ifdef EMMALLOC_DEBUG_LOG
-  EM_ASM({ Module.print("  emmalloc.alignedAllocation") });
+  EM_ASM({ out("  emmalloc.alignedAllocation") });
 #endif
   assert(alignment > ALIGNMENT);
   assert(alignment % ALIGNMENT == 0);
@@ -962,7 +962,7 @@ static void* alignedAllocation(size_t size, size_t alignment) {
   // No luck from free list, so do a new allocation which we can
   // force to be aligned.
 #ifdef EMMALLOC_DEBUG_LOG
-  EM_ASM({ Module.print("    emmalloc.alignedAllocation new allocation") });
+  EM_ASM({ out("    emmalloc.alignedAllocation new allocation") });
 #endif
   // Ensure a region before us, which we may enlarge as necessary.
   if (!lastRegion) {
@@ -1038,7 +1038,7 @@ EMMALLOC_EXPORT
 void* malloc(size_t size) {
 #ifdef EMMALLOC_DEBUG
 #ifdef EMMALLOC_DEBUG_LOG
-  EM_ASM({ Module.print("emmalloc.malloc " + $0) }, size);
+  EM_ASM({ out("emmalloc.malloc " + $0) }, size);
 #endif
   emmalloc_validate_all();
 #ifdef EMMALLOC_DEBUG_LOG
@@ -1048,7 +1048,7 @@ void* malloc(size_t size) {
   void* ptr = emmalloc_malloc(size);
 #ifdef EMMALLOC_DEBUG
 #ifdef EMMALLOC_DEBUG_LOG
-  EM_ASM({ Module.print("emmalloc.malloc ==> " + $0) }, ptr);
+  EM_ASM({ out("emmalloc.malloc ==> " + $0) }, ptr);
 #endif
 #ifdef EMMALLOC_DEBUG_LOG
   emmalloc_dump_all();
@@ -1062,7 +1062,7 @@ EMMALLOC_EXPORT
 void free(void *ptr) {
 #ifdef EMMALLOC_DEBUG
 #ifdef EMMALLOC_DEBUG_LOG
-  EM_ASM({ Module.print("emmalloc.free " + $0) }, ptr);
+  EM_ASM({ out("emmalloc.free " + $0) }, ptr);
 #endif
   emmalloc_validate_all();
 #ifdef EMMALLOC_DEBUG_LOG
@@ -1082,7 +1082,7 @@ EMMALLOC_EXPORT
 void* calloc(size_t nmemb, size_t size) {
 #ifdef EMMALLOC_DEBUG
 #ifdef EMMALLOC_DEBUG_LOG
-  EM_ASM({ Module.print("emmalloc.calloc " + $0) }, size);
+  EM_ASM({ out("emmalloc.calloc " + $0) }, size);
 #endif
   emmalloc_validate_all();
 #ifdef EMMALLOC_DEBUG_LOG
@@ -1092,7 +1092,7 @@ void* calloc(size_t nmemb, size_t size) {
   void* ptr = emmalloc_calloc(nmemb, size);
 #ifdef EMMALLOC_DEBUG
 #ifdef EMMALLOC_DEBUG_LOG
-  EM_ASM({ Module.print("emmalloc.calloc ==> " + $0) }, ptr);
+  EM_ASM({ out("emmalloc.calloc ==> " + $0) }, ptr);
 #endif
 #ifdef EMMALLOC_DEBUG_LOG
   emmalloc_dump_all();
@@ -1106,7 +1106,7 @@ EMMALLOC_EXPORT
 void* realloc(void *ptr, size_t size) {
 #ifdef EMMALLOC_DEBUG
 #ifdef EMMALLOC_DEBUG_LOG
-  EM_ASM({ Module.print("emmalloc.realloc " + [$0, $1]) }, ptr, size);
+  EM_ASM({ out("emmalloc.realloc " + [$0, $1]) }, ptr, size);
 #endif
   emmalloc_validate_all();
 #ifdef EMMALLOC_DEBUG_LOG
@@ -1116,7 +1116,7 @@ void* realloc(void *ptr, size_t size) {
   void* newPtr = emmalloc_realloc(ptr, size);
 #ifdef EMMALLOC_DEBUG
 #ifdef EMMALLOC_DEBUG_LOG
-  EM_ASM({ Module.print("emmalloc.realloc ==> " + $0) }, newPtr);
+  EM_ASM({ out("emmalloc.realloc ==> " + $0) }, newPtr);
 #endif
 #ifdef EMMALLOC_DEBUG_LOG
   emmalloc_dump_all();
@@ -1130,7 +1130,7 @@ EMMALLOC_EXPORT
 int posix_memalign(void **memptr, size_t alignment, size_t size) {
 #ifdef EMMALLOC_DEBUG
 #ifdef EMMALLOC_DEBUG_LOG
-  EM_ASM({ Module.print("emmalloc.posix_memalign " + [$0, $1, $2]) }, memptr, alignment, size);
+  EM_ASM({ out("emmalloc.posix_memalign " + [$0, $1, $2]) }, memptr, alignment, size);
 #endif
   emmalloc_validate_all();
 #ifdef EMMALLOC_DEBUG_LOG
@@ -1140,7 +1140,7 @@ int posix_memalign(void **memptr, size_t alignment, size_t size) {
   int result = emmalloc_posix_memalign(memptr, alignment, size);
 #ifdef EMMALLOC_DEBUG
 #ifdef EMMALLOC_DEBUG_LOG
-  EM_ASM({ Module.print("emmalloc.posix_memalign ==> " + $0) }, result);
+  EM_ASM({ out("emmalloc.posix_memalign ==> " + $0) }, result);
 #endif
 #ifdef EMMALLOC_DEBUG_LOG
   emmalloc_dump_all();
@@ -1154,7 +1154,7 @@ EMMALLOC_EXPORT
 void* memalign(size_t alignment, size_t size) {
 #ifdef EMMALLOC_DEBUG
 #ifdef EMMALLOC_DEBUG_LOG
-  EM_ASM({ Module.print("emmalloc.memalign " + [$0, $1]) }, alignment, size);
+  EM_ASM({ out("emmalloc.memalign " + [$0, $1]) }, alignment, size);
 #endif
   emmalloc_validate_all();
 #ifdef EMMALLOC_DEBUG_LOG
@@ -1164,7 +1164,7 @@ void* memalign(size_t alignment, size_t size) {
   void* ptr = emmalloc_memalign(alignment, size);
 #ifdef EMMALLOC_DEBUG
 #ifdef EMMALLOC_DEBUG_LOG
-  EM_ASM({ Module.print("emmalloc.memalign ==> " + $0) }, ptr);
+  EM_ASM({ out("emmalloc.memalign ==> " + $0) }, ptr);
 #endif
 #ifdef EMMALLOC_DEBUG_LOG
   emmalloc_dump_all();
@@ -1178,7 +1178,7 @@ EMMALLOC_EXPORT
 struct mallinfo mallinfo() {
 #ifdef EMMALLOC_DEBUG
 #ifdef EMMALLOC_DEBUG_LOG
-  EM_ASM({ Module.print("emmalloc.mallinfo") });
+  EM_ASM({ out("emmalloc.mallinfo") });
 #endif
   emmalloc_validate_all();
 #ifdef EMMALLOC_DEBUG_LOG
