@@ -1,9 +1,9 @@
 import os, shutil, logging
 
-TAG = 'version_44'
+TAG = 'version_50'
 
 def needed(settings, shared, ports):
-  if not settings.BINARYEN: return False
+  if not settings.WASM: return False
   try:
     if shared.BINARYEN_ROOT: # if defined, and not falsey, we don't need the port
       logging.debug('binaryen root already set to ' + shared.BINARYEN_ROOT)
